@@ -15,6 +15,19 @@ aveLag <- function(x, lag=1){
   }
 }
 
+Scale <- function(x){
+  SD <- sd(x,na.rm=TRUE)
+  if (length(SD) == 0 || is.na(SD)){
+    SD <- 0
+  }
+  if (SD == 0){
+    x[] <- 0
+    return(x)
+  } else {
+    return((x - mean(x,na.rm=TRUE)) / SD)
+  }
+}
+
 
 mlVAR <- function(
   data, # Data frame
@@ -29,9 +42,9 @@ mlVAR <- function(
   
   # Estimation options:
   # orthogonal, # TRUE or FALSE for orthogonal edges. Defaults to nvar < 6
-  estimator = c("lmer"), # Add more? estimator = "least-squares" IGNORES multi-level
-  # contemporaneous = c("default","shared","unique"), # Shared or unique contamporaneous relationships?
-  temporal = c("default", "correlated","orthogonal","fixed"), # Unique = multi-level!
+  estimator = c("default","lmer","lm"), # Add more? estimator = "least-squares" IGNORES multi-level
+  contemporaneous = c("default","correlated","orthogonal","fixed","unique"), # IF NOT FIXED: 2-step estimation method (lmer again on residuals)
+  temporal = c("default", "correlated","orthogonal","fixed","unique"), # Unique = multi-level!
   # betweenSubjects = c("default","GGM","posthoc"), # Should covariances between means be estimated posthoc or as a GGM? Only used when method = "univariate"
   
   # Misc:
@@ -40,11 +53,16 @@ mlVAR <- function(
   # JAGSoptions = list(),
   verbose = TRUE, # Include progress bar?
   compareToLags,
+  scale = TRUE, # standardize variables grand mean before estimation
+  
   orthogonal # Used for backward competability
+  
+  # nCores = 1,
   # JAGSexport = FALSE, # Exports jags files
   # n.chain = 3,
   # n.iter = 10000,
   # estOmega = FALSE
+  # ...
 )
 {
   if (0 %in% lags & length(lags) > 1){
@@ -72,16 +90,14 @@ mlVAR <- function(
   # Some dummies for later versions:
   # temporal <- "unique"
   temporal <- match.arg(temporal)
-  contemporaneous <- "shared"
+  contemporaneous <- match.arg(contemporaneous)
   
   if (!missing(orthogonal)){
     temporal <- ifelse(orthogonal,"orthogonal","correlated")
     warning(paste0("'orthogonal' argument is deprecated Setting temporal = '",temporal,"'"))
   }
 #   
-#   if (contemporaneous == "default"){
-#     contemporaneous <- ifelse(estimator == "least-squares", "unique", "shared")
-#   }
+
 #   if (betweenSubjects != "default"){
 #     if (estimator != "lmer"){
 #       warning("'betweenSubjects' argument not used in estimator.")
@@ -123,15 +139,55 @@ mlVAR <- function(
   }
   
   # Set orthogonal:
+  if (estimator == "default"){
+    if (temporal == "unique"){
+      estimator <- "lm"
+    } else {
+      estimator <- "lmer"
+    }
+    if (verbose) {
+      message(paste0("'estimator' argument set to '",estimator,"'"))
+    }
+  }
+  
   if (temporal == "default"){
     if (length(vars) > 6){
       
-      if (verbose) message("More than 6 nodes, correlations between random effects are set to zero (temporal = 'orthogonal')")
       temporal <- "orthogonal"
       
     } else {
       temporal <- "correlated"
-    }    
+    }  
+    if (verbose) {
+      message(paste0("'temporal' argument set to '",temporal,"'"))
+    }
+  }
+  
+  if (contemporaneous == "default"){
+    if (length(vars) > 6){
+      
+      contemporaneous <- "orthogonal"
+      
+    } else {
+      contemporaneous <- "correlated"
+    }  
+    
+    if (verbose) {
+      message(paste0("'contemporaneous' argument set to '",contemporaneous,"'"))
+    }
+  }
+  
+  ### Check input ###
+  if (estimator == "lmer"){
+    if (temporal %in% c("unique")){
+      stop("'lmer' estimator does not support temporal = 'unique'")
+    }
+  }
+  
+  if (estimator == "lm"){
+    if (!temporal %in% c("unique")){
+      stop("'lm' estimator only supports temporal = 'unique'")
+    }
   }
   
   # CompareToLags:
@@ -174,8 +230,45 @@ mlVAR <- function(
   # Remove NA day or beeps:
   data <- data[!is.na(data[[idvar]]) & !is.na(data[[dayvar]]) & !is.na(data[[beepvar]]), ]
   
-  ### Codes from mlVAR
-  # Create mlVAR-like predictor data-frame:
+  ### STANDARDIZE DATA ###
+  # Test for rank-deficient:
+  X <- as.matrix(na.omit(data[,vars]))
+  qrX <- qr(X)
+  rnk <- qrX$rank
+  
+  if (rnk < length(vars)){
+    # Which node is not being nice?
+    keep <- qrX$pivot[1:rnk]
+    discard <- vars[!seq_along(vars)%in%keep]
+    
+    warning(paste0("The following variables are linearly dependent on other columns, and therefore dropped from the mlVAR analysis:\n",
+                   paste("\t-",discard,collapse="\n")))
+  
+    # If only one, we can find it out:
+    if (rnk == length(vars) - 1){
+      drop <- qrX$pivot[length(qrX$pivot)]
+      test <- try(qr.solve(X[,-drop],X[,drop]))
+      if (!is(test,"try-error")){
+        test <- round(test,4)
+        test <- test[test!=0]
+        msg <- paste0(discard," = ",paste(test," * ",names(test),collapse=" + "))
+        warning(msg)
+      }
+    }
+    
+    vars <- vars[keep]
+    
+  }
+  
+  if (scale){
+    for (v in vars){
+      data[[v]] <- Scale(data[[v]])
+    }
+  }
+ 
+  
+  ### Codes from murmur
+  # Create murmur-like predictor data-frame:
   # Within-subjects model:
   PredModel <- expand.grid(
     dep = vars,
@@ -186,7 +279,7 @@ mlVAR <- function(
   )
   
    # Between-subjects model:
-  if (betweenSubjects == "GGM" & estimator != "JAGS"){
+  if (betweenSubjects == "GGM" & estimator == "lmer"){
     between <- expand.grid(dep=vars,pred=vars,lag=NA,type="between",
                           stringsAsFactors = FALSE)
     
@@ -234,14 +327,16 @@ mlVAR <- function(
       # between: add mean variable:
       
       if (UniquePredModel$type[i] == "between"){
-        augData[[UniquePredModel$predID[i]]] <- ave(augData[[UniquePredModel$pred[i]]],augData[[idvar]], FUN = aveMean)
+        if (estimator == "lmer"){
+          augData[[UniquePredModel$predID[i]]] <- ave(augData[[UniquePredModel$pred[i]]],augData[[idvar]], FUN = aveMean)          
+        }
       } else {
         # First include:
         augData[[UniquePredModel$predID[i]]] <-  ave(augData[[UniquePredModel$pred[i]]],augData[[idvar]],augData[[dayvar]], FUN = function(x)aveLag(x,UniquePredModel$lag[i]))
         
         # Then center:
         ### CENTERING ONLY NEEDED WHEN ESTIMATOR != JAGS ###
-        if (!estimator %in% c("stan","JAGS")){
+        if (!estimator %in% c("JAGS")){
           augData[[UniquePredModel$predID[i]]] <- ave(augData[[UniquePredModel$predID[i]]],augData[[idvar]], FUN = aveCenter) 
         } 
       }
@@ -257,9 +352,12 @@ mlVAR <- function(
   
   #### RUN THE MODEL ###
   if (estimator == "lmer"){
-    Res <- lmer_mlVAR(PredModel,augData,idvar,verbose=verbose, betweenSubjects=betweenSubjects,temporal=temporal)
-  # } else if (estimator == "least-squares"){
-    # Res <- leastSquares_mlVAR(PredModel,augData,idvar,orthogonal = orthogonal, verbose=verbose, betweenSubjects=betweenSubjects)
+    Res <- lmer_mlVAR(PredModel,augData,idvar,verbose=verbose, contemporaneous=contemporaneous,temporal=temporal)
+  # } else if (estimator == "stan"){
+    # Res <- stan_mlVAR(PredModel,augData,idvar,verbose=verbose,temporal=temporal,nCores=nCores,...)    
+  
+    } else if (estimator == "lm"){
+    Res <- lm_mlVAR(PredModel,augData,idvar,temporal=temporal,contemporaneous=contemporaneous, verbose=verbose)
 #   } else if (estimator == "JAGS"){
 #     Res <- JAGS_mlVAR(augData, vars, 
 #                       idvar,
